using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace HyttysPeli;

/// @author salmatel
/// @version 12.11.2023
/// <summary>
/// 
/// </summary>
public class HyttysPeli : PhysicsGame
{
    Image taustaKuva = LoadImage("taustakuva");
    public override void Begin()
    {
        MasterVolume = 0.05;
        SetWindowSize(1920, 1080, true);
        Level.Background.Image = taustaKuva;
        
        BoundingRectangle alaosa = new BoundingRectangle(new Vector(-100, 1), Level.BoundingRect.BottomRight);
        BoundingRectangle ylaosa = new BoundingRectangle(new Vector(-960, 0), new Vector(960, 540));
        
        
        PhysicsObject pelaaja = new PhysicsObject(25, 100, Shape.Rectangle);
        pelaaja.Position = RandomGen.NextVector(alaosa);
        pelaaja.Image = LoadImage("hyttyskarkoite");
        Add(pelaaja);
        
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä avustus");
        Keyboard.Listen(Key.Up, ButtonState.Down, LyoHyttysta, "Pelaaja ylös", pelaaja, new Vector(0, 20));
        Keyboard.Listen(Key.Down, ButtonState.Down, LyoHyttysta, "Pelaaja alas", pelaaja, new Vector(0, -20));
        Keyboard.Listen(Key.Left, ButtonState.Down, LyoHyttysta, "Pelaaja vasemmalle", pelaaja, new Vector(-20, 0));
        Keyboard.Listen(Key.Right, ButtonState.Down, LyoHyttysta, "Pelaaja oikealle", pelaaja, new Vector(20, 0));


        for (int i = 0; i < 25; i++)
        {
            LuoHyttynen(this, ylaosa, 60, "hyttynen");
        }

        for (int j = 0; j < 10; j++)
        {
            LuoAmpiainen(this, ylaosa, 60, "ampiainen");
        }
        
        AddCollisionHandler(pelaaja, "hyttynen", PelaajaTormasiHyttyseen);
        AddCollisionHandler(pelaaja, "ampiainen", PelaajaTormasiAmpiaiseen);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
   
    
    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja hyttynen törmäävät.
    /// Luodaan hyttysen kohdalla räjähdys ja poistetaan hyttynen pelistä
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="hyttynen">hyttynen johon pelaaja törmäsi</param>
    private void PelaajaTormasiHyttyseen(PhysicsObject pelaaja, PhysicsObject hyttynen)
    {
        Explosion rajahdys = new Explosion(hyttynen.Width * 2);
        rajahdys.Position = hyttynen.Position;
        rajahdys.UseShockWave = false; 
        this.Add(rajahdys);
        Remove(hyttynen);
    }


    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja joku ampiainen törmäävät.
    /// Jos pelaaja osuu ampiaiseen, peli alkaa alusta.
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="ampiainen">ampiainen johon pelaaja törmäsi</param>
    private void PelaajaTormasiAmpiaiseen(PhysicsObject pelaaja, PhysicsObject ampiainen)
    {
        ClearAll();
        Begin();
    }
    
    
    /// <summary>
    /// Lyödään hyttystä voimavektorilla
    /// </summary>
    /// <param name="hyttynen">lyötävä hyttynen</param>
    /// <param name="suunta">voimavektori</param>
    public static void LyoHyttysta(PhysicsObject hyttynen, Vector suunta)
    { 
        hyttynen.Hit(hyttynen.Mass * suunta);
    }

    
    /// <summary>
    /// Luodaan satunnainen hyttynen.
    /// </summary>
    /// <param name="peli">Mihin peliin hyttynen luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Hyttysen vauhti alkutilanteessa</param>
    /// <param name="tunniste">Hyttyselle annettava tunniste</param>
    /// <returns>Viite luotuun hyttyseen</returns>
    public PhysicsObject LuoHyttynen(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = 50;
        double korkeus = 50;
        PhysicsObject hyttynen = new PhysicsObject(leveys, korkeus, Shape.Rectangle);
        hyttynen.Position = RandomGen.NextVector(rect);
        Vector suunta = RandomGen.NextVector(0, vauhti);
        hyttynen.Hit(suunta);
        hyttynen.Tag = tunniste;
        hyttynen.Image = LoadImage("hyttynen (2)");
        peli.Add(hyttynen);
        return hyttynen;
    }
    
    
    /// <summary>
    /// Luodaan satunnainen ampiainen.
    /// </summary>
    /// <param name="peli">Mihin peliin ampiainen luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Ampiaisen vauhti alkutilanteessa</param>
    /// <param name="tunniste">Ampiaiselle annettava tunniste</param>
    /// <returns>Viite luotuun Ampiaiseen</returns>
    public PhysicsObject LuoAmpiainen(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = 75;
        double korkeus = 75;
        PhysicsObject ampiainen = new PhysicsObject(leveys, korkeus, Shape.Rectangle);
        ampiainen.Position = RandomGen.NextVector(rect);
        Vector suunta = RandomGen.NextVector(0, vauhti);
        ampiainen.Hit(suunta);
        ampiainen.Tag = tunniste;
        ampiainen.Image = LoadImage("ampiainen");
        peli.Add(ampiainen);
        return ampiainen;
    }
}
